# This file was auto-generated by Fern from our API Definition.

import typing

Action = typing.Union[
    typing.Literal[
        "key",
        "type",
        "mouse_move",
        "left_click",
        "left_click_drag",
        "right_click",
        "middle_click",
        "double_click",
        "screenshot",
        "cursor_position",
    ],
    typing.Any,
]
